@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE OverloadedStrings #-}
@@ -81,6 +82,7 @@ import Data.XML.Types
     )
 import Data.Typeable (Typeable)
 import Data.Data (Data)
+import Control.DeepSeq(NFData(rnf))
 import Data.Text (Text)
 import qualified Text.XML.Stream.Parse as P
 import qualified Text.XML.Unresolved as D
@@ -125,6 +127,11 @@ data Document = Document
     }
   deriving (Show, Eq, Typeable, Data)
 
+#if MIN_VERSION_containers(0, 4, 2)
+instance NFData Document where
+  rnf (Document a b c) = rnf a `seq` rnf b `seq` rnf c `seq` ()
+#endif
+
 data Node
     = NodeElement Element
     | NodeInstruction Instruction
@@ -132,6 +139,14 @@ data Node
     | NodeComment Text
   deriving (Show, Eq, Ord, Typeable, Data)
 
+#if MIN_VERSION_containers(0, 4, 2)
+instance NFData Node where
+  rnf (NodeElement e) = rnf e `seq` ()
+  rnf (NodeInstruction i) = rnf i `seq` ()
+  rnf (NodeContent t) = rnf t `seq` ()
+  rnf (NodeComment t) = rnf t `seq` ()
+#endif
+
 data Element = Element
     { elementName :: Name
     , elementAttributes :: Map.Map Name Text
@@ -139,6 +154,11 @@ data Element = Element
     }
   deriving (Show, Eq, Ord, Typeable, Data)
 
+#if MIN_VERSION_containers(0, 4, 2)
+instance NFData Element where
+  rnf (Element a b c) = rnf a `seq` rnf b `seq` rnf c `seq` ()
+#endif
+
 {-
 readFile :: FilePath -> ParseSettings -> IO (Either SomeException Document)
 readFile_ :: FIlePath -> ParseSettings -> IO Document
